#ifndef __AbstractionFunction_h__
#define __AbstractionFunction_h__

#include "Map.h"
#include "String.h"

class AbstractionFunction
{
public:
    void add( String &concrete, String &abstract )
    {
        _concreteToAbstract[concrete] = abstract;

        if ( !_abstractToConcrete.exists( abstract ) )
        {
            Vector<String> newVector;
            _abstractToConcrete[abstract] = newVector;
        }

        _abstractToConcrete[abstract].append( concrete );
    }

    String concreteToAbstract( const String &concrete ) const
    {
        return _concreteToAbstract.get( concrete );
    }

    Vector<String> abstractToConcrete( const String &abstract ) const
    {
        return _abstractToConcrete.get( abstract );
    }

    void dump()
    {
        printf( "Dumpting abstraction function\n" );

        Map<String,Vector<String> >::iterator it;

        for ( it = _abstractToConcrete.begin(); it != _abstractToConcrete.end(); ++it )
        {
            printf( "Abstract state: %s\n", it->first.ascii() );
            printf( "Concrete states:\n" );

            for ( unsigned i = 0; i < it->second.size(); ++i )
            {
                printf( "\t%s\n", it->second[i].ascii() );
            }

            printf( "\n" );
        }
    }

    void deleteAbstractState( const String &abstractState )
    {
        _abstractToConcrete.erase( abstractState );
    }

private:
    Map<String,String> _concreteToAbstract;
    Map<String,Vector<String> > _abstractToConcrete;
};

#endif // __AbstractionFunction_h__

//
// Local Variables:
// compile-command: "make -C .. "
// tags-file-name: "../TAGS"
// c-basic-offset: 4
// End:
//
