#ifndef __HttpServer_h__
#define __HttpServer_h__

#include "BProgram.h"
#include "CustomizedOracle.h"
#include "HttpDispatcher.h"
#include "HttpErrorHandler.h"
#include "HttpFoundHandler.h"
#include "HttpGetHandler.h"
#include "Logger.h"
#include "Oracle.h"
#include "SocketManager.h"

class HttpServer
{
public:
    static void start()
    {
        HttpServer().run();
    }

    void run()
    {
        updateEventSelection();
        runEngine();

        try
        {
            runProgram();
            sleepForever();
        }
        catch ( const Error &e )
        {
            reportError( e );
        }
    }

private:
    BProgram _bProgram;
    SocketManager _socketManager;
    //    Logger _logger;
    HttpDispatcher _httpDispatcher;
    HttpGetHandler _httpGetHandler;
    HttpErrorHandler _httpErrorHandler;
    HttpFoundHandler _httpFoundHandler;

    void runEngine()
    {
        _bProgram.runEngine();
    }

    void runProgram()
    {
        _bProgram.addThread( _socketManager );
        //        _bProgram.addThread( _logger );
        _bProgram.addThread( _httpDispatcher );
        _bProgram.addThread( _httpGetHandler );
        _bProgram.addThread( _httpErrorHandler );
        _bProgram.addThread( _httpFoundHandler );

        _bProgram.runProgram();
    }

    void sleepForever()
    {
        enum {
            FOREVER = 10000,
        };

        sleep( FOREVER );
    }

    void reportError( const Error &e )
    {
        printf( "Main Thread: Error caught. Code: %u. Errno: %i\n", e.code(), e.errno() );
        fflush( 0 );
    }

    void updateEventSelection()
    {
        Oracle::setChooser( new CustomizedOracle );
    }
};

#endif // __HttpServer_h__

//
// Local Variables:
// compile-command: "make -C .. "
// tags-file-name: "../TAGS"
// c-basic-offset: 4
// End:
//
