#ifndef __TcpDispatcher_h__
#define __TcpDispatcher_h__

#include "BThread.h"
#include "IEvent.h"
#include "TcpSegment.h"

class TcpDispatcher : public BThread
{
public:
    void entryPoint()
    {
        while ( true )
        {
            waitForRequests();
            classifyRequest();
        }
    }

private:
    void waitForRequests()
    {
        clearEventVectors();
        _watched.append( IEvent::TCP_DATA_RECEIVED );
        _blocked = allRelevantEventsExcept( IEvent::TCP_DATA_RECEIVED );
        bSync( _requested, _watched, _blocked, "Wait" );
    }

    void classifyRequest()
    {
        TcpSegment tcpSegment;
        tcpSegment.unserialize( lastEvent().payload() );

        if ( tcpSegment.header().getDestinationPort() != ServerConfiguration::LISTENING_PORT )
        {
            doneHandlingIpPacket();
            return;
        }

        if ( tcpSegment.header().getSyn() )
        {
            synRequest();
        }
        else if ( tcpSegment.header().getFin() )
        {
            finRequest();
        }
        else if ( tcpSegment.header().getPush() )
        {
            handlePushedData();
        }
        else
        {
            doneHandlingIpPacket();
        }
    }

    void synRequest()
    {
        clearEventVectors();
        _requested.append( Event( IEvent::TCP_SYN_REQUEST, lastEvent().payload() ) );
        _blocked = allRelevantEventsExcept( IEvent::TCP_SYN_REQUEST );

        bSync( _requested, _watched, _blocked, "Syn" );
    }

    void finRequest()
    {
        clearEventVectors();
        _requested.append( Event( IEvent::TCP_FIN_REQUEST, lastEvent().payload() ) );
        _blocked = allRelevantEventsExcept( IEvent::TCP_FIN_REQUEST );

        bSync( _requested, _watched, _blocked, "Fin" );
    }

    void doneHandlingIpPacket()
    {
        clearEventVectors();
        _requested.append( Event( IEvent::TCP_DONE_HANDLING_DATA ) );
        _blocked = allRelevantEventsExcept( IEvent::TCP_DONE_HANDLING_DATA );
        bSync( _requested, _watched, _blocked, "Unknown" );
    }

    void handlePushedData()
    {
        clearEventVectors();
        _requested.append( Event( IEvent::TCP_PUSH_REQUEST, lastEvent().payload() ) );
        _blocked = allRelevantEventsExcept( IEvent::TCP_PUSH_REQUEST );
        bSync( _requested, _watched, _blocked, "Push" );
    }

    Vector<Event> allRelevantEventsExcept( Event event )
    {
        Vector<Event> result;

        if ( event != IEvent::TCP_SYN_REQUEST ) result.append( IEvent::TCP_SYN_REQUEST );
        if ( event != IEvent::TCP_FIN_REQUEST ) result.append( IEvent::TCP_FIN_REQUEST );
        if ( event != IEvent::TCP_PUSH_REQUEST ) result.append( IEvent::TCP_PUSH_REQUEST );
        if ( event != IEvent::TCP_DATA_RECEIVED ) result.append( IEvent::TCP_DATA_RECEIVED );

        return result;
    }
};

#endif // __TcpDispatcher_h__

//
// Local Variables:
// compile-command: "make -C .. "
// tags-file-name: "../TAGS"
// c-basic-offset: 4
// End:
//
