#ifndef __TcpDispatcherSpanner_h__
#define __TcpDispatcherSpanner_h__

#include "TcpDispatcher.h"
#include "StateGraphSpanner.h"

class TcpDispatcherToken : public StateGraphSpanner::ThreadToken
{
public:
    BThread *freshCopy()
    {
        return new TcpDispatcher;
    }
};

class TcpDispatcherSpanner
{
public:
    void eventsForTcpDispatcher( Vector<Event> &events )
    {
        events.clear();
        events = IEvent::allEventsExcept( IEvent::TCP_DATA_RECEIVED );

        HeapData serialized;

        TcpSegment syn;
        syn.header().setDestinationPort( ServerConfiguration::LISTENING_PORT );
        syn.header().setSyn();
        syn.serialize( serialized );
        events.append( Event( IEvent::TCP_DATA_RECEIVED, serialized ) );

        TcpSegment fin;
        fin.header().setDestinationPort( ServerConfiguration::LISTENING_PORT );
        fin.header().setFin();
        fin.serialize( serialized );
        events.append( Event( IEvent::TCP_DATA_RECEIVED, serialized ) );

        TcpSegment push;
        push.header().setDestinationPort( ServerConfiguration::LISTENING_PORT );
        push.header().setPush();
        push.serialize( serialized );
        events.append( Event( IEvent::TCP_DATA_RECEIVED, serialized ) );

        TcpSegment unknown;
        unknown.header().setDestinationPort( ServerConfiguration::LISTENING_PORT + 1 );
        unknown.serialize( serialized );
        events.append( Event( IEvent::TCP_DATA_RECEIVED, serialized ) );
    }

    void span( ThreadStateGraph &stateGraph )
    {
        eventsForTcpDispatcher( stateGraph._events );

        TcpDispatcherToken token;
        StateGraphSpanner().span( token, stateGraph );

        stateGraph._events.clear();
        stateGraph._events = IEvent::allEvents();
        stateGraph._threadName = "tcpDispatcher";
    }
};

#endif // __TcpDispatcherSpanner_h__

//
// Local Variables:
// compile-command: "make -C ../.. "
// tags-file-name: "../../TAGS"
// c-basic-offset: 4
// End:
//
